
(require 'string)

(defcustom textlus-books-directory "~/books" "The directory with text files books")
(defcustom textlus-synth-command "RHVoice -r 1.4 | aplay 2> /dev/null" "The command to invoke TTS engine to produce speech")

(defvar textlus-files nil "List of file names in textlus buffer")
(defvar textlus-last-line nil "The cursor position in textlus buffer before last reading operation")
(defvar textlus-mode-map nil "Key map for list of text books")

(let ((map (make-sparse-keymap)))
  (suppress-keymap map t)
  (define-key map "\r" 'textlus-run)
  (setq textlus-mode-map map))

(defun textlus-prepare-string-for-shell (s)
  (when s
    (let ((r))
      (if (string-match "\'" s)
	(setq r (string-replace-match "\'" s "\'\\\'\'" t t))
	(setq r s))
      (format "\'%s\'"  r))))

(defun textlus-fill-initial ()
  (setq buffer-read-only nil)
  (erase-buffer)
  (setq textlus-files (split-string (shell-command-to-string (format "find %s -iname \'*.txt\' 2> /dev/null | sort" (textlus-prepare-string-for-shell (file-truename textlus-books-directory)))) "\n"))
  (let ((q) (n))
    (setq q textlus-files)
    (while q
      (setq n (file-name-nondirectory (car q)))
      (if (string-match "\\.txt" n)
	(setq n (string-replace-match "\\.txt" n "" t t)))
      (when (> (length n) 0)
	(insert n "\n"))
      (setq q (cdr q))))
  (setq buffer-read-only t)
  (goto-char 1))

(defun textlus ()
  "List of text books to listen with Textlus utility"
  (interactive)
  (set-buffer (get-buffer-create "textlus"))
  (switch-to-buffer "textlus")
  (textlus-fill-initial)
  (setq major-mode 'textlus-mode)
  (setq mode-name "textlus-mode")
  (use-local-map textlus-mode-map))

(defun textlus-run ()
  "Starts reading of the text book"
  (interactive)
  (setq textlus-last-line (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0)))
  (let ((ln) (q))
    (setq ln textlus-last-line)
    (setq ln (- ln 1))
    (setq q textlus-files)
    (while (not (equal ln 0))
      (setq q (cdr q))
      (setq ln (- ln 1)))
    (setq q (car q))
    (when (> (length q) 0)
      (shell-command-to-string (format "textlus --synth %s --output none --bookmark %s 2> /dev/null" (textlus-prepare-string-for-shell textlus-synth-command) (textlus-prepare-string-for-shell q))))))

(provide 'textlus)